<?php

namespace App\Http\Controllers\Api\V1\Auth\Customer;

use App\Base\WalletRemarks;
use App\Events\AndroidPushNotification;
use App\Events\CustomerRegistered;
use App\Http\Controllers\Controller;
use App\Http\Requests\Api\CustomerRegistrationRequest;
use App\Models\City;
use App\Models\Customer;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;

class CustomerRegisterController extends Controller
{

    protected Customer $customer;

    public function __construct(Customer $customer)
    {
        $this->customer = $customer;
    }


    public function register(CustomerRegistrationRequest $request)
    {
        $city_request = $request->input('city');
        $city_id = City::query()->where('id', $city_request)->first();

        if (!$city_id) {
            return responseJson(false, trans('Unable to find City'));
        }

        $validate_exists_email = $this->customer->where('email', $request->email)->exists();


        if ($validate_exists_email) {
            return responseJson(false, trans('Provided email has already been taken'));
        }

        $mobile = $request->mobile;

        $validate_exists_mobile = $this->customer->where('mobile', $mobile)->exists();

        if ($validate_exists_mobile) {
            return responseJson(false, trans('Provided mobile has already been taken'));
        }

        if ($request->has('referral_code')) {
            // Validate Referral code
            $referred_customer_record = $this->customer->where('referral_code', $request->referral_code)->first();
            if (!$referred_customer_record) {
                return responseJson(false, trans('Provided Referral code is not valid'));
            }

            // Add referral commission to the referred customer
            $this->addCommissionToReferredCustomer($referred_customer_record);
        }

        $image = null;

        if ($request->hasFile('image')) {
            try {
                $image = uploadImage($request->image, imagePath()['profile']['customer']['path']);
            } catch (\Exception $e) {
                return responseJson(false, trans('api.image_not_uploaded'));
            }
        }

        DB::beginTransaction();
        try {
            $customer_params = [
                'name' => $request->input('name'),
                'email' => $request->input('email'),
                'mobile' => $mobile,
                'address' => $request->input('address'),
                'fcm_token' => $request->input('device_token'),
                'login_by' => $request->input('login_by'),
                'city_id' => $request->input('city'),
                'gender' => $request->input('gender'),
                'referral_code' => str_random(6),
                'image' => $image,
            ];

            if ($request->has('password') && $request->input('password')) {
                $customer_params['password'] = Hash::make($request->input('password'));
            }
            $customer = $this->customer->create($customer_params);


//        $customer->attachRole(Role::Customer);

            event(new CustomerRegistered($customer));

            DB::commit();
        } catch (\Exception $e) {
            DB::rollBack();
            Log::error($e);
            Log::error('Error while Registering a user account. Input params : ' . json_encode($request->all()));
            return responseJson(false, trans('api.something_wrong'));
        }

        if ($customer) {
            return authenticateAndRespond($request, $customer);
        }

        return responseJson(false, trans('api.something_wrong'));
    }

    /**
     * Add Commission to the referred customer
     */
    public function addCommissionToReferredCustomer($referred_customer)
    {
        $referral_commission = settings()->get('referral_commission_for_customer') ?: 0;

        $referred_customer->balance += $referral_commission;
        $referred_customer->save();

        // Add the history
        $referred_customer->transactions()->create([
            'amount' => $referral_commission,
            'trx' => str_random(10),
            'trx_type' => '+',
            'remarks' => WalletRemarks::REFERRAL_COMMISSION,
            'referral_code' => $referred_customer->referral_code,
        ]);

        // Notify user
        $title = trans('api.referral_earnings_notify_title');
        $body = trans('api.referral_earnings_notify_body');
        $fcm_token = $referred_customer->fcm_token;
        event(new AndroidPushNotification($title, $body, $fcm_token));
    }
}
