<?php

namespace App\Http\Controllers\Api\V1\Auth\Vendor;

use App\Http\Controllers\Controller;
use App\Http\Resources\VendorResource;
use App\Models\Vendor;
use App\Traits\GeneralTrait;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;

class AuthController extends Controller
{
    use GeneralTrait;
    /**
     * Create a new AuthController instance.
     *
     * @return void
     */
    public function __construct() {
        $this->middleware('auth:vendor_api', ['except' => ['login', 'register']]);
    }
    /**
     * Get a JWT via given credentials.
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function login(Request $request){
        $validator = Validator::make($request->all(), [
            'email' => 'required|email',
            'password' => 'required|string|min:6',
        ]);
        if ($validator->fails()) {
            return $this->returnValidationError('442', $validator);
        }
        if (! $token = auth()->guard('vendor_api')->attempt($validator->validated())) {
            return $this->returnError('401', 'Unauthorized');
        }
            return $this->createNewToken($token);
    }
    /**
     * Register a User.
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function register(Request $request) {
        $validator = Validator::make($request->all(), [
            'name'      => 'required|string|between:2,100',
            'email'     => 'required|string|email|max:100|unique:users',
            'password'  => 'required|string|min:8',
        ]);
        if($validator->fails()){
            return $this->returnValidationError('400', $validator);
        }
        $vendor = Vendor::create(array_merge(
            $validator->validated(),
            ['password' => Hash::make($request->password)]
        ));
       return $this->returnData('data', $vendor, 'Vendor successfully registered');
    }

    /**
     * Log the user out (Invalidate the token).
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function logout() {
        auth()->guard('vendor_api')->logout();
       return $this->returnSuccessMessage('Vendor successfully signed out', '200');
    }
    /**
     * Refresh a token.
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function refresh() {
        return $this->createNewToken(auth()->guard('vendor_api')->refresh());
    }
    /**
     * Get the authenticated User.
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function userProfile() {
        $vendor =   new VendorResource(auth()->guard('vendor_api')->user());
        return response()->json($vendor);
    }
    /**
     * Get the token array structure.
     *
     * @param  string $token
     *
     * @return \Illuminate\Http\JsonResponse
     */
    protected function createNewToken($token){
        return response()->json([
            'access_token' => $token,
            'token_type' => 'bearer',
            'expires_in' => auth()->guard('vendor_api')->factory()->getTTL() * 60,
            'data' => auth()->guard('vendor_api')->user()
        ]);
    }
}
