<?php

use App\Events\CustomerLogin;
use App\Http\Resources\CustomerDataResource;
use App\Models\CustomerEmailLog;
use App\Models\EmailTemplate;
use Carbon\Carbon;
use Illuminate\Http\JsonResponse;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Str;
use Intervention\Image\ImageManager;
use PHPMailer\PHPMailer\PHPMailer;

if (!function_exists('getActiveRoutesHome')) {
    function getActiveRoutesHome($route)
    {
        $actives = request()->routeIs($route) ? ' active' : null;
        return $actives;
    }
}

if (!function_exists('responseJson')) {
    function responseJson($success, $message, $data = null, $model = null): JsonResponse
    {
        return response()->json([
            'success' => $success,
            'message' => $message,
            'data' => $data,
            'paginate' => $model,
        ]);
    }
}

if (!function_exists('responseWithPagination')) {
    function responseWithPagination($model): array
    {
        return [
            'total' => $model->total(),
            'per_page' => $model->perPage(),
            'current_page' => $model->currentPage(),
            'last_page' => $model->lastPage(),
            'total_pages' => $model->lastPage(),
            'from' => $model->firstItem(),
            'to' => $model->lastItem(),
            "next_page_url" => $model->nextPageUrl(),
            "prev_page_url" => $model->previousPageUrl(),
            "path" => $model->path(),
        ];
    }
}

if (!function_exists('uploadImage')) {

    function uploadImage($file, $location, $size = null, $old = null, $thumb = null): string
    {
        $path = makeDirectory($location);
        if (!$path) return responseJson(false, trans('api.file_could_not_been_created'));

        if ($old) {
            removeFile($location . '/' . $old);
            removeFile($location . '/thumb_' . $old);
        }
        $filename = uniqid() . time() . '.' . $file->getClientOriginalExtension();
        $image = ImageManager::imagick()->read($file);
        if ($size) {
            $size = explode('x', strtolower($size));
            $image->resize($size[0], $size[1]);
        }
        $image->save($location . '/' . $filename);

        if ($thumb) {
            $thumb = explode('x', $thumb);
            ImageManager::imagick()->read($file)->resize($thumb[0], $thumb[1])->save($location . '/thumb_' . $filename);
        }

        return $filename;
    }
}

if (!function_exists('uploadFile')) {
    function uploadFile($file, $location, $size = null, $old = null): string
    {
        $path = makeDirectory($location);
        if (!$path) throw new Exception('File could not been created.');

        if ($old) {
            removeFile($location . '/' . $old);
        }

        $filename = uniqid() . time() . '.' . $file->getClientOriginalExtension();
        $file->move($location, $filename);
        return $filename;
    }
}

if (!function_exists('makeDirectory')) {
    function makeDirectory($path): bool
    {
        if (file_exists($path)) return true;
        return mkdir($path, 0755, true);
    }
}

if (!function_exists('removeFile')) {
    function removeFile($path): bool
    {
        return file_exists($path) && is_file($path) && @unlink($path);
    }
}

if (!function_exists('imagePath')) {
    function imagePath(): array
    {
        $data['image'] = [
            'default' => 'upload/default.png',
        ];
        $data['language'] = [
            'path' => 'upload/lang',
            'size' => '64x64'
        ];
        $data['logo'] = [
            'path' => 'upload/settings',
        ];
        $data['favicon'] = [
            'path' => 'upload/settings',
            'size' => '128x128',
        ];
        $data['profile'] = [
            'default' => 'upload/avatar.jpg',
            'admin' => [
                'path' => 'upload/admin/profile',
                'size' => '400x400'
            ],
            'customer' => [
                'path' => 'upload/customer/profile',
                'size' => '350x300'
            ],
            'vendor' => [
                'path' => 'upload/vendor/profile',
                'size' => '350x300'
            ],
            'specialist' => [
                'path' => 'upload/merchant/profile',
                'size' => '350x300'
            ],
            'owner' => [
                'path' => 'upload/owner/profile',
                'size' => '350x300'
            ],
        ];
        $data['product'] = [
            'path' => 'upload/product',
            'size' => '400x270',
            'thumb' => '280x189'
        ];
        $data['advertisement'] = [
            'path' => 'upload/advertisement',
        ];
        $data['country'] = [
            'path' => 'upload/country',
        ];
        $data['categories'] = [
            'path' => 'upload/categories',
            'size' => '200x200',
        ];
        return $data;
    }
}

if (!function_exists('str_random')) {
    function str_random($length = 16): string
    {
        return Str::random($length);
    }
}

if (!function_exists('ip')) {

    function ip(): ?string
    {
        return request()->ip();
    }
}

if (!function_exists('verificationCode')) {

    function verificationCode($length): int
    {
        if ($length == 0) return 0;
        $min = pow(10, $length - 1);
        $max = 0;
        while ($length > 0 && $length--) {
            $max = ($max * 10) + 9;
        }
        return random_int($min, $max);
    }
}

if (!function_exists('shortCodeReplacer')) {
    function shortCodeReplacer($shortCode, $replace_with, $template_string): array|string
    {
        return str_replace($shortCode, $replace_with, $template_string);
    }
}

if (!function_exists('sendEmail')) {

    /**
     * @throws Exception
     */
    function sendEmail($user, $type = null, $shortCodes = [], $userType = 'customer'): void
    {

        $emailTemplate = EmailTemplate::where('act', $type)->where('status', 1)->first();
        if (!$emailTemplate) {
            return;
        }

        $logo = asset(imagePath()['logo']['path'] . '/' . settings()->get('logo'));
        $year = Carbon::now()->year;

        $message = shortCodeReplacer("{{year}}", $year, settings()->get('email_template'));
        $message = shortCodeReplacer("{{logo}}", $logo, $message);
        $message = shortCodeReplacer("{{app_url}}", settings()->get('app_url'), $message);
        $message = shortCodeReplacer("{{app_name}}", settings()->get('app_name'), $message);
        $message = shortCodeReplacer("{{name}}", $user->name, $message);
        $message = shortCodeReplacer("{{email}}", $user->email, $message);
        $message = shortCodeReplacer("{{message}}", $emailTemplate->body, $message);


        if (empty($message)) {
            $message = $emailTemplate->email_body;
        }

        foreach ($shortCodes as $code => $value) {
            $message = shortCodeReplacer('{{' . $code . '}}', $value, $message);
        }

        $config = json_decode(settings()->get('mail_config'));

        if ($userType == 'vendor') {
            $emailLog = new VendorEmailLog();
            $emailLog->vendor_id = $user->id;
        } else {
            $emailLog = new CustomerEmailLog();
            $emailLog->customer_id = $user->id;
        }

        $emailLog->mail_sender = $config->name;
        $emailLog->mail_from = settings()->get('app_name') . ' ' . settings()->get('email_from');
        $emailLog->mail_to = $user->email;
        $emailLog->subject = $emailTemplate->subj;
        $emailLog->message = $message;
        $emailLog->save();


        if ($config->name == 'php') {
            sendPhpMail($user->email, $user->name, $emailTemplate->subj, $message);
        } else {
            sendSmtpMail($config, $user->email, $user->name, $emailTemplate->subj, $message);
        }
    }
}

if (!function_exists('sendPhpMail')) {
    function sendPhpMail($receiver_email, $receiver_name, $subject, $message): void
    {
        $app_name = settings()->get('app_name');
        $email_from = settings()->get('email_from');
        $headers = "From: $app_name <$email_from> \r\n";
        $headers .= "Reply-To: $app_name <$email_from> \r\n";
        $headers .= "MIME-Version: 1.0\r\n";
        $headers .= "Content-Type: text/html; charset=utf-8\r\n";
        @mail($receiver_email, $subject, $message, $headers);
    }
}

if (!function_exists('sendSmtpMail')) {
    function sendSmtpMail($config, $receiver_email, $receiver_name, $subject, $message)
    {
        $app_name = settings()->get('app_name');
        $email_from = settings()->get('email_from');
        $mail = new PHPMailer(true);

        try {
            //Server settings
            $mail->isSMTP();
            $mail->Host = $config->host;
            $mail->SMTPAuth = true;
            $mail->Username = $config->username;
            $mail->Password = $config->password;
            if ($config->enc == 'ssl') {
                $mail->SMTPSecure = PHPMailer::ENCRYPTION_SMTPS;
            } else {
                $mail->SMTPSecure = PHPMailer::ENCRYPTION_STARTTLS;
            }
            $mail->Port = $config->port;
            $mail->CharSet = 'UTF-8';
            //Recipients
            $mail->setFrom($email_from, $app_name);
            $mail->addAddress($receiver_email, $receiver_name);
            $mail->addReplyTo($email_from, $app_name);
            // Content
            $mail->isHTML(true);
            $mail->Subject = $subject;
            $mail->Body = $message;
            $mail->send();
        } catch (Exception $e) {
            throw new Exception($e);
        }
    }
}

if (!function_exists('authenticateAndRespond')) {

    function authenticateAndRespond($request, $user = null, $guard = ApiGuard::CUSTOMER_API): JsonResponse
    {
        $remember = $request->boolean('remember');

        if ($user != null) {
            $access_token = auth($guard)->login($user, $remember);
        } else {
            $credentials = $request->only(['password']);
            $loginType = filter_var($request->input('login'), FILTER_VALIDATE_EMAIL) ? 'email' : 'mobile';
            if (!$access_token = auth()->guard($guard)->attempt(array_merge([$loginType => $request->login],$credentials), $remember)) {
                return responseJson(false, trans('auth.failed'));
            }
        }

        $user = auth()->guard($guard)->user();

        if ($guard == ApiGuard::VENDOR_API) {
            event(new VendorAuth($user));
            $data = new VendorDataResource($user);
        } else {
            event(new CustomerAuth($user));
            $data = new CustomerDataResource($user);
        }



        $response = [
            'user' => $data,
            'token_type' => 'Bearer',
            'access_token' => $access_token,
        ];

        return responseJson(true, trans('api.login_successfully'), $response);
    }

    if (!function_exists('getIpInfo')) {
        function getIpInfo(): array
        {
            $ip = $_SERVER["REMOTE_ADDR"];

            //Deep detect ip
            if (filter_var(@$_SERVER['HTTP_X_FORWARDED_FOR'], FILTER_VALIDATE_IP)) {
                $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
            }
            if (filter_var(@$_SERVER['HTTP_CLIENT_IP'], FILTER_VALIDATE_IP)) {
                $ip = $_SERVER['HTTP_CLIENT_IP'];
            }


            $xml = @simplexml_load_file("http://www.geoplugin.net/xml.gp?ip=" . $ip);


            $country = @$xml->geoplugin_countryName;
            $city = @$xml->geoplugin_city;
            $area = @$xml->geoplugin_areaCode;
            $code = @$xml->geoplugin_countryCode;
            $long = @$xml->geoplugin_longitude;
            $lat = @$xml->geoplugin_latitude;

            $data['country'] = $country;
            $data['city'] = $city;
            $data['area'] = $area;
            $data['code'] = $code;
            $data['long'] = $long;
            $data['lat'] = $lat;
            $data['ip'] = request()->ip();
            $data['time'] = date('d-m-Y h:i:s A');


            return $data;
        }
    }

    if (!function_exists('osBrowser')) {
        function osBrowser(): array
        {
            $userAgent = $_SERVER['HTTP_USER_AGENT'];
            $osPlatform = "Unknown OS Platform";
            $osArray = array(
                '/windows nt 10/i' => 'Windows 10',
                '/windows nt 6.3/i' => 'Windows 8.1',
                '/windows nt 6.2/i' => 'Windows 8',
                '/windows nt 6.1/i' => 'Windows 7',
                '/windows nt 6.0/i' => 'Windows Vista',
                '/windows nt 5.2/i' => 'Windows Server 2003/XP x64',
                '/windows nt 5.1/i' => 'Windows XP',
                '/windows xp/i' => 'Windows XP',
                '/windows nt 5.0/i' => 'Windows 2000',
                '/windows me/i' => 'Windows ME',
                '/win98/i' => 'Windows 98',
                '/win95/i' => 'Windows 95',
                '/win16/i' => 'Windows 3.11',
                '/macintosh|mac os x/i' => 'Mac OS X',
                '/mac_powerpc/i' => 'Mac OS 9',
                '/linux/i' => 'Linux',
                '/ubuntu/i' => 'Ubuntu',
                '/iphone/i' => 'iPhone',
                '/ipod/i' => 'iPod',
                '/ipad/i' => 'iPad',
                '/android/i' => 'Android',
                '/blackberry/i' => 'BlackBerry',
                '/webos/i' => 'Mobile'
            );
            foreach ($osArray as $regex => $value) {
                if (preg_match($regex, $userAgent)) {
                    $osPlatform = $value;
                }
            }
            $browser = "Unknown Browser";
            $browserArray = array(
                '/msie/i' => 'Internet Explorer',
                '/firefox/i' => 'Firefox',
                '/safari/i' => 'Safari',
                '/chrome/i' => 'Chrome',
                '/edge/i' => 'Edge',
                '/opera/i' => 'Opera',
                '/netscape/i' => 'Netscape',
                '/maxthon/i' => 'Maxthon',
                '/konqueror/i' => 'Konqueror',
                '/mobile/i' => 'Handheld Browser'
            );
            foreach ($browserArray as $regex => $value) {
                if (preg_match($regex, $userAgent)) {
                    $browser = $value;
                }
            }

            $data['os_platform'] = $osPlatform;
            $data['browser'] = $browser;

            return $data;
        }
    }
}


if (!function_exists('include_route_files')) {
    /**
     * Loops through a folder and requires all PHP files
     * Searches sub-directories as well.
     *
     * @param string $folder
     */
    function include_route_files($folder): void
    {
        $path = base_path('routes' . DIRECTORY_SEPARATOR . $folder);
        $rdi = new recursiveDirectoryIterator($path);
        $it = new recursiveIteratorIterator($rdi);

        while ($it->valid()) {
            if (!$it->isDot() && $it->isFile() && $it->isReadable() && $it->current()->getExtension() === 'php') {
                require $it->key();
            }

            $it->next();
        }
    }
}

if (!function_exists('custom_encryption')) {
    function custom_encryption($text, $length = 15): string
    {
        return Crypt::encryptString(str_pad($text, $length, '0', STR_PAD_LEFT));
    }
}

